#!/usr/bin/env python3

import functools
import logging
import os
import random
import shutil
import tempfile
import unittest

import requests

import canalplus


class TestCanalPlus(unittest.TestCase):

  def checkIsVideo(self, video, *, download=True):
    self.assertIsInstance(video, canalplus.CanalPlusVideo)
    self.assertIsInstance(video.title, str)
    self.assertTrue(video.title)
    self.assertTrue(video.stream_url.startswith("http"))
    if video.stream_url.endswith(".m3u8"):
      m3u_data = requests.get(video.stream_url).text
      streams = tuple(canalplus.CanalPlusVideo.parseM3U(m3u_data))
      self.assertGreaterEqual(len(streams), 1)
      for stream in streams:
        if stream[0].endswith(".m3u8"):
          self.assertGreaterEqual(len(streams), 3)  # 3 qualities
          m3u_data = requests.get(stream[0]).text
          media_streams = tuple(canalplus.CanalPlusVideo.parseM3U(m3u_data))
          # each ts file is 10s long, assume no video is shorter than 30s
          self.assertGreaterEqual(len(media_streams), 3)
    if download:
      with tempfile.TemporaryDirectory() as temp_dir_path:
        video.download(temp_dir_path)
        downloaded = tuple(map(functools.partial(os.path.join, temp_dir_path),
                               os.listdir(temp_dir_path)))
        self.assertEqual(len(downloaded), 1)
        self.assertTrue(os.path.isfile(downloaded[0]))
        self.assertGreaterEqual(os.path.getsize(downloaded[0]), 0)
    if shutil.which("true") is not None:
      video.view("true")

  def test_getProgramList(self):
    """ Get program list, check expected programs and download a video. """
    programs = canalplus.CanalPlusProgramList()
    self.assertGreaterEqual(len(programs), 40)
    for program in programs:
      self.assertIsInstance(program, canalplus.CanalPlusProgram)
      #self.assertTrue(program)
      self.assertIsInstance(program.title, str)
      self.assertTrue(program.title)
    self.assertIn("divertissement", programs)
    self.assertIn("BANDES ANNONCES FILMS DIFFUSES SUR CANAL+", programs)
    if False:  # slow!
      for program in programs:
        for video in program:
          video.fetchVideoUrl()
          self.checkIsVideo(video, download=False)
    else:
      programs = tuple(programs)
      program = None
      while not program:
        program = random.choice(programs)
      video = random.choice(tuple(program))
      video.fetchVideoUrl()
      self.checkIsVideo(video)

  def test_search(self):
    """ Search for a program, and download a search result. """
    videos = canalplus.CanalPlusSearch("les guignols")
    videos = tuple(videos)
    self.assertGreaterEqual(len(videos), 40)
    video = random.choice(videos)
    video.fetchVideoUrl()
    self.checkIsVideo(video)


if __name__ == "__main__":
  # disable logging
  logging.basicConfig(level=logging.CRITICAL + 1)
  #logging.basicConfig(level=logging.DEBUG)

  # run tests
  unittest.main()



import abc
import itertools
import shutil
import time


class Progress(metaclass=abc.ABCMeta):

  """ Progress abstract interface. """

  def __init__(self, *, max_updates_per_sec=10):
    self._current_progress = 0
    self._additionnal_info = None
    self._max_updates_per_sec = max_updates_per_sec
    self._last_update_time = 0

  def updateProgress(self, progress):
    """ Change progress percentage value. """
    if (progress < 0) or (progress > 100):
      raise ValueError()
    self._current_progress = progress

  def setAdditionnalInfo(self, info):
    """ Add additionnal text to be displayed with the progress. """
    self._additionnal_info = info

  def display(self):
    """ Display the progress if needed. """
    # short circuit if displaying too fast
    now = time.monotonic()
    if ((self._current_progress != 100) and
            (self._max_updates_per_sec > 0) and
            ((now - self._last_update_time) <= (1 / self._max_updates_per_sec))):
      return
    self._display()
    self._last_update_time = now

  @abc.abstractmethod
  def end(self):
    """ End display. """
    pass

  @abc.abstractmethod
  def _display(self):
    """ Display the progress. """
    pass


class ProgressBar(Progress):

  """ Terminal progress bar. """

  def __init__(self, *, max_updates_per_sec=10, append_eta=False):
    super().__init__(max_updates_per_sec=max_updates_per_sec)
    self._line_with = shutil.get_terminal_size(fallback=(80, 0))[0] - 1
    self._append_eta = append_eta
    self._start_time = None

  def updateProgress(self, progress):
    if self._append_eta and (self._start_time is None):
      self._start_time = time.monotonic()
    super().updateProgress(progress)

  def _display(self):
    """ See Progress._display. """
    bar_width = self._line_with - 8
    if self._append_eta:
      now = time.monotonic()
      eta_s = (100 - self._current_progress) * (now - self._start_time) / max(0.01, self._current_progress)
      if eta_s > (60 * 99):
        h = eta_s // (60 * 60)
        m = (eta_s - h * 60 * 60) // 60
        s = eta_s % 60
        eta_str = "ETA %02u:%02u:%02u" % (h, m, s)
      else:
        m = eta_s // 60
        s = eta_s % 60
        eta_str = "ETA %02u:%02u" % (m, s)
      bar_width -= len(eta_str) + 1
    if self._additionnal_info is not None:
      bar_width -= 1 + len(self._additionnal_info)
      additionnal_info = "%s " % (self._additionnal_info)
    else:
      additionnal_info = ""
    char_progress = int(self._current_progress * bar_width / 100)
    if char_progress == 0:
      bar_str = " " * bar_width
    elif char_progress == bar_width:
      bar_str = "=" * bar_width
    else:
      bar_str = ("%s>" % ("=" * (char_progress - 1))).ljust(bar_width)
    line = "\r%s[%3u%%] [%s]" % (additionnal_info, self._current_progress, bar_str)
    if self._append_eta:
      line = " ".join((line, eta_str))
    print(line, end="")

  def end(self):
    """ See Progress.end. """
    print()


class SimpleProgress(Progress):

  """ Progress percentage display. """

  def __init__(self, *, max_updates_per_sec=10):
    super().__init__(max_updates_per_sec=max_updates_per_sec)

  def _display(self):
    """ See Progress._display. """
    line = "\r%u%%" % (self._current_progress)
    print(line, end="")
    if self._additionnal_info is not None:
      print(" %s" % (self._additionnal_info), end="")

  def end(self):
    """ See Progress.end. """
    print()


class ZenityProgress(SimpleProgress):

  """ Progress producing output to pipe to zenity --progress. """

  def __init__(self, *, max_updates_per_sec=10):
    super().__init__(max_updates_per_sec=max_updates_per_sec)

  def _display(self):
    """ See Progress._display. """
    line = "%u" % (self._current_progress)
    print(line, flush=True)
    if self._additionnal_info is not None:
      print("# %s" % (self._additionnal_info), flush=True)

  def end(self):
    """ See Progress.end. """
    pass


class SimpleAnimatedProgress(Progress):

  """ Progress percentage display with animated scrobbler. """

  def __init__(self, *, max_updates_per_sec=10):
    super().__init__(max_updates_per_sec=max_updates_per_sec)
    self._scrobbler = itertools.cycle("|/-\\")

  def _display(self):
    """ See Progress._display. """
    line = "\r%s %u%%" % (next(self._scrobbler), self._current_progress)
    print(line, end="")
    if self._additionnal_info is not None:
      print(" %s" % (self._additionnal_info), end="")

  def end(self):
    """ See Progress.end. """
    print()



#!/usr/bin/env python3

""" Browse, play, or download CANAL+ videos. """

__version__ = "1.0.9"
__author__ = "desbma"
__license__ = "GPLv3"

import argparse
import contextlib
import itertools
import logging
import os
import shutil
import signal
import subprocess
import string
import sys
import urllib.parse
import xml.etree.ElementTree

import requests

from canalplus import colored_logging
from canalplus import mkstemp_ctx
from canalplus import progress_display


USER_AGENT = "Mozilla/5.0"
HTTP_TIMEOUT = 9.1


def format_byte_size_str(size):
  if size > 1000000000:
    return "%0.3fGB" % (size / 1000000000)
  elif size > 1000000:
    return "%0.2fMB" % (size / 1000000)
  elif size > 1000:
    return "%uKB" % (size // 1000)
  return "%uB" % (size)


class CanalPlusApiObject:

  """ Base class for Canal+ API objects. """

  BASE_URL = "http://service.canal-plus.com/video/rest"

  session = requests.Session()

  def getHttpSession(self):
    return __class__.session

  def fetchXml(self, action, parameter=""):
    """ Fetch XML data from an URL and return a xml.etree.ElementTree object. """
    url = "%s/%s/cplus/%s" % (self.BASE_URL, action, parameter)
    xml_text = self.fetchText(url)
    return xml.etree.ElementTree.fromstring(xml_text)

  def fetchText(self, url):
    """ Fetch text from an URL. """
    logging.getLogger().debug("Fetching '%s'..." % (url))
    response = self.getHttpSession().get(url,
                                         headers={"User-Agent":
                                                  USER_AGENT},
                                         timeout=HTTP_TIMEOUT)
    response.raise_for_status()
    return response.content.decode("utf-8")


class CanalPlusVideo(CanalPlusApiObject):

  """ Canal+ video API object. """

  def __init__(self, id, title):
    self.id = id
    self.title = title
    self.stream_url = None

  def download(self, dir):
    """ Download a video to a given directory. """
    if self.stream_url is None:
      self.fetchVideoUrl()

    # sanitize output filename
    video_filepath_ts = os.path.join(dir,
                                     "%s.ts" % (self.title.replace("/", "-").strip(string.whitespace + ".")))
    video_filepath_mp4 = "%s.mp4" % (os.path.splitext(video_filepath_ts)[0])

    if os.path.isfile(video_filepath_ts) or os.path.isfile(video_filepath_mp4):
      logging.getLogger().info("File already exists, skipping download")
      return

    try:
      with mkstemp_ctx.mkstemp(suffix=".ts") as video_filepath_tmp:
        # download
        if sys.stdout.isatty() and logging.getLogger().isEnabledFor(logging.INFO):
          progress = progress_display.ProgressBar(append_eta=True)
        else:
          progress = None
        if self.stream_url.endswith(".m3u8"):
          # fetch m3u8 playlist
          m3u8_data = self.fetchText(self.stream_url)
          # parse it
          ts_urls = tuple(filter(lambda x: not x.startswith("#"),
                                 m3u8_data.splitlines()))
          # download ts files
          self.download_ts(ts_urls, video_filepath_tmp, progress)

        else:
          # direct stream download
          logging.getLogger().info("Downloading video to '%s'..." % (video_filepath_ts))
          self.download_ts((self.stream_url,), video_filepath_tmp, progress)

        if progress is not None:
          progress.updateProgress(100)
          progress.display()
          progress.end()

        # try to remux to mp4
        remuxed = self.remuxToMp4(video_filepath_tmp, video_filepath_mp4)
        if not remuxed:
          shutil.move(video_filepath_tmp, video_filepath_ts)

    except Exception as e:
      logging.getLogger().error("Download failed: %s %s", e.__class__.__qualname__, e)
      exit(1)

  def download_ts(self, urls, filepath, progress):
    """ Download one or several MPEG-TS videos to a file. """
    logging.getLogger().info("Downloading TS file%s..." % ("s" if len(urls) > 1 else ""))
    with open(filepath, "wb") as video_file:
      for i, ts_url in enumerate(urls):
        previous_size = video_file.tell()
        with contextlib.closing(self.getHttpSession().get(ts_url,
                                                          stream=True,
                                                          headers={"User-Agent":
                                                                   USER_AGENT},
                                                          timeout=HTTP_TIMEOUT)) as response:
          response.raise_for_status()
          ts_size = int(response.headers["Content-Length"])
          for chunk in response.iter_content(2 ** 12):
            if progress is not None:
              total_dl_bytes = video_file.tell()
              ts_dl_bytes = total_dl_bytes - previous_size
              progress.updateProgress((i * 100 / len(urls)) +
                                      ts_dl_bytes * (100 / len(urls)) / ts_size)
              progress.setAdditionnalInfo("TS file %s/%u: %s / %s, total %s" %
                                          (str(i + 1).rjust(len(str(len(urls)))),
                                           len(urls),
                                           format_byte_size_str(ts_dl_bytes).rjust(6),
                                           format_byte_size_str(ts_size).rjust(6),
                                           format_byte_size_str(total_dl_bytes).rjust(7)))
              progress.display()
            video_file.write(chunk)

  def remuxToMp4(self, ts_filepath, mp4_filepath):
    """ Remux TS file to MP4, return True if success, false instead. """
    ffmpeg_path = shutil.which("ffmpeg")
    avconv_path = shutil.which("avconv")
    remuxed = False
    if ffmpeg_path is not None or avconv_path is not None:
      # remux to mp4 (better seeking than mpegts)
      converter = os.path.basename(next(filter(None, (ffmpeg_path, avconv_path))))
      logging.getLogger().info("Remuxing to '%s' with %s..." % (mp4_filepath, converter))
      for attempt in range(2):
        cmd = [converter]
        if not logging.getLogger().isEnabledFor(logging.DEBUG):
          cmd.extend(("-loglevel", "quiet"))
        cmd.extend(("-i", ts_filepath, "-c", "copy"))
        if attempt == 1:
          cmd.extend(("-bsf:a", "aac_adtstoasc"))
        cmd.append(mp4_filepath)
        try:
          subprocess.check_call(cmd)
        except subprocess.CalledProcessError:
          try:
            os.remove(mp4_filepath)
          except FileNotFoundError:
            pass
          continue
        remuxed = True
        os.remove(ts_filepath)
        break
      if not remuxed:
        logging.getLogger().warning("Remuxing failed")
    return remuxed

  def view(self, player):
    """ View a video in a given media player. """
    logger = logging.getLogger()
    if self.stream_url is None:
      self.fetchVideoUrl()
    logger.info("Viewing in player '%s'..." % (player))
    player_cmd = (player, self.stream_url)
    if logger.isEnabledFor(logging.DEBUG):
      subprocess.check_call(player_cmd)
    else:
      subprocess.check_call(player_cmd,
                            stdin=subprocess.DEVNULL,
                            stdout=subprocess.DEVNULL,
                            stderr=subprocess.DEVNULL)

  def fetchVideoUrl(self):
    """ Fetch video URL for the best quality available. """
    # get video infos
    logging.getLogger().info("Getting video metadata...")
    xml_vidinfo = self.fetchXml("getVideos", self.id)
    playlist_url = xml_vidinfo.findtext("VIDEO/MEDIA/VIDEOS/HLS")
    if playlist_url:
      playlist = self.fetchText(playlist_url)
      self.stream_url = self.getPlaylistBestQuality(playlist)
    else:
      self.stream_url = xml_vidinfo.findtext("VIDEO/MEDIA/VIDEOS/HD")
      if not self.stream_url:
        self.stream_url = xml_vidinfo.findtext("VIDEO/MEDIA/VIDEOS/HAUT_DEBIT")
      if not self.stream_url:
        self.stream_url = xml_vidinfo.findtext("VIDEO/MEDIA/VIDEOS/BAS_DEBIT")
      assert(self.stream_url)

  def getPlaylistBestQuality(self, playlist):
    """ Parse an M3U8 playlist content, and return best quality stream. """
    highest_bitrate = 0
    best_quality_url = None
    streams = __class__.parseM3U(playlist)
    for url, attribs in streams:
      current_bitrate = int(attribs.partition("BANDWIDTH=")[2].split(",", 1)[0])
      logging.getLogger().debug("Got bitrate of %u" % (current_bitrate))
      if current_bitrate > highest_bitrate:
        highest_bitrate = current_bitrate
        best_quality_url = url
    assert(best_quality_url is not None)
    return best_quality_url

  @staticmethod
  def parseM3U(data):
    """ Parse M3U data and yield tuples of (url, attrib line str). """
    attrib_prefix = "#EXT-X-STREAM-INF"
    lines = data.splitlines()
    if lines[0] != "#EXTM3U":
      # not valid m3u
      return None
    attribs = None
    for line in lines[1:]:
      if line.startswith(attrib_prefix):
        attribs = line[len(attrib_prefix):]
      elif line.startswith("#"):
        # ignore
        continue
      else:
        # url
        yield (line, attribs)
        attribs = None


class CanalPlusProgram(CanalPlusApiObject):

  """ Canal+ program (iterable of CanalPlusVideo) API object. """

  def __init__(self, id, title):
    self.id = id
    self.title = title
    self.xml_vidlist = None

  def __iter__(self):
    """ Get an iterator over program videos. """
    if self.xml_vidlist is None:
      self.fetchVidlist()
    return self.__next__()

  def __next__(self):
    """ Get a video. """
    for xml_vid in self.xml_vidlist.iterfind("MEA"):
      id = int(xml_vid.findtext("ID"))
      title = xml_vid.findtext("INFOS/TITRAGE/TITRE")
      subtitle = xml_vid.findtext("INFOS/TITRAGE/SOUS_TITRE")
      if subtitle:
        title = "%s (%s)" % (title, subtitle)
      yield CanalPlusVideo(id, title)

  def __getitem__(self, index):
    """ Get a video from the program at a given index. """
    if self.xml_vidlist is None:
      self.fetchVidlist()
    xml_vid = self.xml_vidlist.findall("MEA")[index]
    id = int(xml_vid.findtext("ID"))
    title = xml_vid.findtext("INFOS/TITRAGE/TITRE")
    subtitle = xml_vid.findtext("INFOS/TITRAGE/SOUS_TITRE")
    if subtitle:
      title = "%s (%s)" % (title, subtitle)
    return CanalPlusVideo(id, title)

  def __bool__(self):
    """ Return True if there is at least one video in the program, False otherwise. """
    if self.xml_vidlist is None:
      self.fetchVidlist()
    return self.xml_vidlist.find("MEA") is not None

  def __len__(self):
    """ Return the number of videos in the program. """
    if self.xml_vidlist is None:
      self.fetchVidlist()
    return len(self.xml_vidlist.findall("MEA"))

  def fetchVidlist(self):
    """ Fetch program video list. """
    # get videos list
    logging.getLogger().info("Getting video list...")
    self.xml_vidlist = self.fetchXml("getMEAs", self.id)


class CanalPlusSearch(CanalPlusApiObject):

  """ Canal+ search result (iterable of CanalPlusVideo) API object. """

  def __init__(self, query):
    self.query = query
    self.fetchVidlist()

  def __iter__(self):
    """ Get an iterator over search results. """
    return self.__next__()

  def __next__(self):
    """ Get a search result video. """
    for xml_vid in self.xml_vidlist.iterfind("VIDEO"):
      id = int(xml_vid.findtext("ID"))
      title = xml_vid.findtext("INFOS/TITRAGE/TITRE")
      subtitle = xml_vid.findtext("INFOS/TITRAGE/SOUS_TITRE")
      if subtitle:
        title = "%s (%s)" % (title, subtitle)
      yield CanalPlusVideo(id, title)

  def __getitem__(self, index):
    """ Get a video search result from a given index. """
    xml_vid = self.xml_vidlist.findall("VIDEO")[index]
    id = int(xml_vid.findtext("ID"))
    title = xml_vid.findtext("INFOS/TITRAGE/TITRE")
    subtitle = xml_vid.findtext("INFOS/TITRAGE/SOUS_TITRE")
    if subtitle:
      title = "%s (%s)" % (title, subtitle)
    return CanalPlusVideo(id, title)

  def __bool__(self):
    """ Return True if there is at least one search result, False otherwise. """
    return self.xml_vidlist.find("VIDEO") is not None

  def __len__(self):
    """ Return the number of search results. """
    return len(self.xml_vidlist.findall("VIDEO"))

  def fetchVidlist(self):
    """ Fetch search results list. """
    # get videos list
    logging.getLogger().info("Getting search results...")
    self.xml_vidlist = self.fetchXml("search", urllib.parse.quote_plus(self.query))


class CanalPlusProgramList(CanalPlusApiObject):

  """ Canal+ program list (iterable of CanalPlusProgram) API object. """

  def __init__(self):
    # get program list
    logging.getLogger().info("Getting program list...")

    xml_programs = self.fetchXml("initPlayer")
    # parse it
    self.programs = {}
    for xml_program_group in xml_programs.iterfind("THEMATIQUES/THEMATIQUE"):
      for xml_program in xml_program_group.iterfind("SELECTIONS/SELECTION"):
        id = int(xml_program.findtext("ID"))
        if id not in self.programs.keys():
          title = xml_program.findtext("NOM")
          self.programs[id] = title

  def __iter__(self):
    """ Get an iterator over programs. """
    return self.__next__()

  def __next__(self):
    """ Get a program. """
    for (id, title) in self.programs.items():
      yield CanalPlusProgram(id, title)

  def __len__(self):
    """ Get the number of programs. """
    return len(self.programs)

  def __contains__(self, title):
    """ Return True if a program with the given title is available, False otherwise. """
    for p in self:
      if p.title.lower() == title.lower():
        return True
    return False

  def __getitem__(self, key):
    """ Get a program from name or index. """
    if isinstance(key, str):
      for p in self:
        if p.title.lower() == key.lower():
          return p
    else:
      return next(itertools.islice(self, key, key + 1))


def terminal_choice(items, autocap=False):
  for i, item in enumerate(items, 1):
    print("% 3u. %s" % (i, string.capwords(item.title) if autocap else item.title))
  c = 0
  while c not in range(1, len(items) + 1):
    try:
      c = int(input("? "))
    except ValueError:
      continue
    except KeyboardInterrupt:
      exit(128 + signal.SIGINT)
  return c


def cl_main():
  # parse args
  arg_parser = argparse.ArgumentParser(description=__doc__,
                                       formatter_class=argparse.ArgumentDefaultsHelpFormatter)
  arg_parser.add_argument("output",
                          help="Output directory to put downloaded files. Use 'player:vlc' to stream in a player.")
  arg_parser.add_argument("-m",
                          "--mode",
                          choices=("auto", "last", "manual"),
                          default="manual",
                          dest="mode",
                          help="What to do with a program (download/view all videos, download/view only last video, \
                                interactively download/view a video")
  arg_parser.add_argument("-p",
                          "--program",
                          default=None,
                          dest="program",
                          help="Program (case insensitive). Use '?program' to do a search instead of looking for an \
                                exact match.")
  arg_parser.add_argument("-v",
                          "--verbose",
                          action="store_true",
                          default=False,
                          dest="verbose",
                          help="Increase program output")
  args = arg_parser.parse_args()

  # setup logger
  logger = logging.getLogger()
  if args.verbose:
    logger.setLevel(logging.DEBUG)
  else:
    logger.setLevel(logging.INFO)
  logging.getLogger("requests").setLevel(logging.ERROR)
  logging.getLogger("urllib3").setLevel(logging.ERROR)
  logging_formatter = colored_logging.ColoredFormatter(fmt="%(message)s")
  logging_handler = logging.StreamHandler()
  logging_handler.setFormatter(logging_formatter)
  logger.addHandler(logging_handler)

  # choose program
  if args.program is None:
    # interactive program selection mode
    programs = CanalPlusProgramList()
    c = terminal_choice(programs, True)
    program = programs[c - 1]
  elif args.program.startswith("?"):
    # program search mode
    program = CanalPlusSearch(args.program[1:])
  else:
    # exact program match mode
    programs = CanalPlusProgramList()
    if args.program in programs:
      program = programs[args.program]
    else:
      logger.error("Unknown program '%s'" % (args.program))
      exit(1)

  # choose vid(s) to download
  if args.mode == "auto":
    # auto mode
    if isinstance(program, CanalPlusProgram):
      logger.info("[Automatic mode] Getting all videos of program '%s'" % (program.title))
    else:
      logger.info("[Automatic mode] Getting all videos for query '%s'" % (program.query))
    for i, vid in enumerate(program, 1):
      logger.info("[Automatic mode] Getting video %u/%u : '%s'" % (i, len(program), vid.title))
      if args.output.startswith("player:"):
        vid.view(args.output.split(":", 1)[1])
      else:
        vid.download(args.output)
  elif args.mode == "last":
    # last video mode
    vid = next(iter(program))
    if isinstance(program, CanalPlusProgram):
      logger.info("[Last video mode] Getting last video of program '%s': '%s'" % (program.title, vid.title))
    else:
      logger.info("[Last video mode] Getting first search result for query '%s': '%s'" % (program.query, vid.title))
    if args.output.startswith("player:"):
      vid.view(args.output.split(":", 1)[1])
    else:
      vid.download(args.output)
  else:
    # interactive mode
    if not program:
      if isinstance(program, CanalPlusProgram):
        logger.error("No videos for program '%s'" % (program.title))
      else:
        logger.error("No videos for search '%s'" % (args.program[1:]))
      exit(1)
    c = terminal_choice(program)
    vid = program[c - 1]
    if args.output.startswith("player:"):
      vid.view(args.output.split(":", 1)[1])
    else:
      vid.download(args.output)


if __name__ == "__main__":
  cl_main()
